package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping, CourtRank}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.collection.mutable

/**
 * @Description:司法案件预处理
 * @author π
 * @date 2020/9/17 14:45
 */
object JudicialCaseRelationPreNew {
  def main(args: Array[String]): Unit = {
    var project = ""
    var ds = ""
    var c = ""
    if (args.length == 3) {
      val Array(p1, p2, p3) = args
      project = p1
      ds = p2
      c = p3
    } else {
      println("please check project ds c!")
      sys.exit(-1)
    }
    if(ds.equals("all")) ds =""
    println(
      s"""
         |project: $project
         |ds: $ds
         |c: $c
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    val r = JudicialCaseRelationPreNew(spark, project, ds)
    c match {
      case "calc" => r.calc()
      case "precalc" => r.precalc()
    }

    spark.stop()
  }
}


case class JudicialCaseRelationPreNew(s: SparkSession, project: String, ds: String
                                     ) extends LoggingUtils with CompanyMapping with BaseFunc with CourtRank {
  override protected val spark: SparkSession = s


  private def getStrToMap(cols: Seq[String]): String = {
    val set = cols.toSet
    val str = set.map(e => {
      s"concat_ws('\001','$e',cast($e as string))"
    }).mkString(",")
    s"str_to_map(concat_ws('\002',$str),'\002','\001')"
  }

  def precalc(): Unit = {
    prepareFunctions(spark)
    val t1 = s"$project.inc_ads_company_court_announcement"
    var t1_ds = ds
    if (StringUtils.isBlank(ds)) {
      t1_ds = BaseUtil.getPartion(t1, spark)
    }

    //法院公告
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_announcement')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |     select
         |     *,row_number() over(partition by detail_id order by date desc) num
         |     from (
         |           select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"2" as flag
         |            ,title(plaintiff,litigant,'法院公告') as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,null as case_reason
         |            ,case_no
         |            ,court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(litigant) as bg_name
         |            ,concat_ws(' ',publish_date,'00:00:00') as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_announcement
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"2" as flag
         |            ,title(plaintiff,litigant,'法院公告') as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,null as case_reason
         |            ,case_no
         |            ,court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(litigant) as bg_name
         |            ,concat_ws(' ',publish_date,'00:00:00') as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_announcement
         |      where length(case_no) > 0 and ds > '0'
         |     )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

    //文书预处理
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='wenshu')
         |SELECT
         |        COALESCE(a.judicase_id,b.new_judicase_id) judicase_id
         |        ,'0' as flag
         |        ,title
         |        ,case_type
         |        ,case_reason
         |        ,case_no
         |        ,court_name
         |        ,concat_ws('',case_stage(case_no)) as case_stage
         |        ,regexp_replace(yg_name,'\n',',') as yg_name
         |        ,regexp_replace(bg_name,'\n',',') as bg_name
         |        ,judge_date as date
         |        ,case_id as detail_id
         |        ,case_amt
         |FROM    (
         |            SELECT  *
         |            FROM $project.ads_judicial_case_relation_graph
         |        ) a
         |RIGHT JOIN (
         |            SELECT *,md5(cleanup(case_no)) as new_judicase_id
         |             FROM
         |            (
         |              SELECT  *,row_number() over(partition by docid order by judge_date desc) num
         |              FROM    $project.ods_wenshu_detail
         |              WHERE   ds > '0'
         |            )c
         |            where num = 1
         |        ) b
         |ON      a.id = b.case_id
         |""".stripMargin).show(10, false)


    //开庭公告预处理
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_open_announcement')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |    select
         |    *,
         |    row_number() over(partition by detail_id order by date desc) num
         |    from
         |    (
         |     select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"1" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(defendant) as bg_name
         |            ,start_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_open_announcement
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"1" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(defendant) as bg_name
         |            ,start_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_open_announcement
         |      where length(case_no) > 0 and ds > '0'
         |    )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)


    //立案信息预处理
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_register')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |     select
         |     *,row_number() over(partition by detail_id order by date desc) num
         |     from (
         |           select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"8" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,plaintiff as yg_name
         |            ,defendant as bg_name
         |            ,filing_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_register
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"8" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,plaintiff as yg_name
         |            ,defendant as bg_name
         |            ,filing_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_register
         |      where length(case_no) > 0 and ds > '0'
         |     )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

  }

  def calc(): Unit = {
    prepareFunctions(spark)
    map_2_json()
    case_no_trim_udf()
    registerCourtRank()
    //预处理数据
    val cols = Seq("flag", "date", "detail_id")

    val t1 = s"$project.inc_ads_company_court_announcement"
    val t2 = s"$project.ads_judicial_case_relation_pre"
    var t2_ds = ds
    var t1_ds = ds
    if (StringUtils.isBlank(ds)) {
      t2_ds = BaseUtil.getPartion(t2, "wenshu", spark)
      t1_ds = BaseUtil.getPartion(t1, spark)
    }
    //司法案件id交换表
    val t3 = "ads_judicial_case_relation_replace"
    val t4 = "ads_judicial_case_incr_mapping"

    //替换司法案件id
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.$t3 partition (ds = '$t1_ds')
         |SELECT  COALESCE(b.judicase_id,a.new_judicase_id) judicase_id
         |        ,a.flag
         |        ,a.title
         |        ,a.case_type
         |        ,a.case_reason
         |        ,case_no_trim(a.case_no) as case_no
         |        ,a.court_name
         |        ,a.case_stage
         |        ,case_label(a.flag) lable
         |        ,map_2_json(${getStrToMap(cols)}) as detail
         |        ,a.yg_name
         |        ,a.bg_name
         |        ,a.date
         |        ,a.detail_id
         |        ,a.case_amt
         |FROM    (
         |  select
         |     judicase_id
         |     ,flag
         |     ,title
         |     ,case_type
         |     ,case_reason
         |     ,case_no_trim(case_no) as case_no
         |     ,court_name
         |     ,case_stage
         |     ,replace_char(yg_name) as yg_name
         |     ,replace_char(bg_name) as bg_name
         |     ,date
         |     ,detail_id
         |     ,case_amt
         |     ,md5(CLEANUP(case_no)) as new_judicase_id
         |  from $project.ads_judicial_case_relation_pre
         |  where ds= '$t2_ds' and tn <> 'wenshu' and case_no_trim(case_no) is not null
         |        and date is not null and length(date) = 19
         |) a
         |LEFT JOIN (
         |  select case_no_trim(case_no) as case_no,max(judicase_id) judicase_id
         |  from $project.ads_judicial_case_relation_pre
         |  where ds = '$t2_ds' and tn ='wenshu' and case_no_trim(case_no) is not null
         |  group by case_no
         |) b
         |ON  CLEANUP(a.case_no) = CLEANUP(b.case_no)
         |union all
         |SELECT   judicase_id
         |        ,flag
         |        ,title
         |        ,case_type
         |        ,case_reason
         |        ,case_no_trim(case_no) as case_no
         |        ,court_name
         |        ,case_stage
         |        ,case_label(flag) lable
         |        ,map_2_json(${getStrToMap(cols)}) as detail
         |        ,replace_char(yg_name) as yg_name
         |        ,replace_char(bg_name) as bg_name
         |        ,date
         |        ,detail_id
         |        ,case_amt
         |from $project.ads_judicial_case_relation_pre
         |where ds = '$t2_ds' and tn ='wenshu'  and case_no_trim(case_no) is not null
         |      and date is not null and length(date) = 19
         |""".stripMargin).show(10, false)

    val second_ds = getSecondLastPartitionOrElse(t3, "0")
    println(s"calc ds: $t2_ds, par ds : $t1_ds, second_ds : $second_ds")


    //找出增量数据
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.$t4
         |SELECT  coalesce(a.judicase_id,b.judicase_id)judicase_id
         |        ,CASE WHEN a.judicase_id IS NULL THEN 1 ELSE 0 END
         |FROM    (
         |            SELECT  judicase_id
         |                    ,md5(concat_ws('',judicase_id, sort(concat_ws('\001',collect_set(case_no))))) r1
         |            FROM    $project.$t3
         |            WHERE   ds = '$t1_ds'
         |            GROUP BY judicase_id
         |        ) a
         |FULL JOIN (
         |              SELECT  judicase_id
         |                      ,md5(concat_ws('',judicase_id, sort(concat_ws('\001',collect_set(case_no))))) r2
         |              FROM    $project.$t3
         |              WHERE   ds = '$second_ds'
         |              GROUP BY judicase_id
         |          ) b
         |ON  r1 = r2
         |WHERE   r1 IS NULL OR r2 IS NULL
         |""".stripMargin)

    //司法案件主表
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci.ads_judicial_case_relation_r1
         |SELECT  judicase_id
         |        ,max(first_title) title
         |        ,max(case_type) case_type
         |        ,max(case_reason) case_reason
         |        ,concat_ws(',',collect_set(case_no)) case_no
         |        ,concat_ws(',',collect_set(court_name)) court_name
         |        ,last_stage(concat_ws(' ',collect_set(case_stage))) case_stage
         |        ,concat_ws(',',max(case_type),collect_set(lable)) lable
         |        ,concat('[',concat_ws(',',collect_set(detail)),']') detail
         |        ,max(first_yg_name) AS yg_name
         |        ,max(first_bg_name) AS bg_name
         |        ,max(case_amt) AS case_amt
         |        ,max(date) AS date
         |        ,trim_black(concat_ws(',',collect_set(court_level))) court_level
         |        ,max(deleted) deleted
         |FROM    (
         |        SELECT  a.* ,first_value(yg_name) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_yg_name
         |                ,first_value(bg_name) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_bg_name
         |                ,first_value(title) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_title
         |                ,b.deleted
         |        FROM    (
         |                   SELECT  *,court_level(court_name) court_level
         |                   FROM    $project.$t3
         |                   WHERE   ds >= '$second_ds'
         |                ) a JOIN
         |                (
         |                   select *
         |                   from $project.$t4
         |                ) b on a.judicase_id = b.judicase_id
         |        )
         |GROUP BY judicase_id
         |""".stripMargin).show(20, false)

    //明细表
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci.ads_judicial_case_relation_r2
         |SELECT  md5(concat_ws('',judicase_id,CLEANUP(case_no),case_stage)) id
         |        ,judicase_id
         |        ,max(first_title) title
         |        ,max(case_type) case_type
         |        ,max(case_reason) case_reason
         |        ,case_no
         |        ,max(court_name) court_name
         |        ,case_stage
         |        ,concat_ws(',',max(case_type),collect_set(lable)) lable
         |        ,concat('[',concat_ws(',',collect_set(detail)),']') detail
         |        ,max(first_yg_name) yg_name
         |        ,max(first_bg_name) bg_name
         |        ,max(last_date) last_date
         |        ,max(deleted) deleted
         |FROM    (
         |        SELECT  a.* ,first_value(yg_name) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_yg_name
         |                ,first_value(bg_name) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_bg_name
         |                ,first_value(title) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_title
         |                ,first_value(date) OVER (PARTITION BY a.judicase_id ORDER BY date DESC ) AS last_date
         |                ,b.deleted
         |        FROM    (
         |                   SELECT  *
         |                   FROM    $project.$t3
         |                   WHERE   ds >= '$second_ds'
         |                )a JOIN
         |                (
         |                   select *
         |                   from $project.$t4
         |                )b on a.judicase_id = b.judicase_id
         |)
         |GROUP BY judicase_id
         |         ,case_no
         |         ,case_stage
         |""".stripMargin).show(10, false)

  }

}