package com.winhc.bigdata.spark.utils

import java.text.SimpleDateFormat
import java.util.regex.Pattern
import java.util.{Calendar, Date, Locale}

import cn.hutool.core.util.StrUtil
import com.winhc.bigdata.spark.implicits.RegexUtils._
import org.apache.commons.lang3.StringUtils
import org.apache.commons.lang3.time.DateFormatUtils
import org.apache.spark.sql.SparkSession

/**
 * @Author: XuJiakai
 * @Date: 2020/6/3 18:49
 * @Description:
 */
object BaseUtil {
  //去其他符号 去空格
  private val pattern = "[^\\u4e00-\\u9fa50-9a-zA-Z]".r

  //去其他符号 去空格  补全null
  def cleanup(s: String): String = {
    if (StringUtils.isBlank(s))
      ""
    else
      pattern replaceAllIn(s, "")
  }

  def isWindows: Boolean = System.getProperty("os.name").contains("Windows")

  def getPartitions(t: String, @transient spark: SparkSession): Seq[String] = {
    import spark._
    val sql_s = s"show partitions " + t
    sql(sql_s).collect.toList.map(_.getString(0).split("=")(1)).seq
  }

  def getPartion(t: String, p: String, @transient spark: SparkSession): String = {
    import spark._
    val sql_s = s"show partitions " + t
    val ps = sql(sql_s).collect.toList.map(r => {
      val Array(x, y) = r.getString(0).split("/")
      val Array(_, a2) = x.split("=")
      val Array(_, b2) = y.split("=")
      var r1 = ("", "")
      if (StringUtils.isNumeric(a2)) {
        r1 = (a2, b2)
      } else {
        r1 = (b2, a2)
      }
      r1
    }).filter(_._2.equals(p)).map(_._1).seq
    if (ps.size > 0) {
      ps.sorted.last
    } else {
      ""
    }
  }

  def getPartion(t: String, @transient spark: SparkSession) = {
    val ps = getPartitions(t, spark)
    if (ps.size > 0) {
      ps.last
    } else {
      ""
    }
  }

  def getFirstPartion(t: String, @transient spark: SparkSession) = {
    val ps = getPartitions(t, spark)
    if (ps.size > 0) {
      ps.head
    } else {
      ""
    }
  }

  def nowDate(pattern: String = "yyyy-MM-dd"): String = {
    new SimpleDateFormat(pattern).format(new Date)
  }

  def atMonthsBefore(n: Int, pattern: String = "yyyy-MM-dd"): String = {
    val c = Calendar.getInstance(Locale.CHINA)
    c.setTimeInMillis(new Date().getTime)
    c.add(Calendar.MONTH, -1 * n)
    DateFormatUtils.format(c.getTime.getTime, pattern)
  }

  def getYesterday(): String = {
    atDaysAfter(-1, nowDate("yyyyMMdd"))
  }

  def getYear(n: Int): String = {
    val cal = Calendar.getInstance(Locale.CHINA)
    cal.add(Calendar.YEAR, 1 * n)
    cal.get(Calendar.YEAR).toString
  }

  def atDaysAfter(n: Int, time: String, pattern: String = "yyyyMMdd"): String = {
    import java.text.SimpleDateFormat
    val newtime: Date = new SimpleDateFormat("yyyyMMdd").parse(time)
    val c = Calendar.getInstance(Locale.CHINA)
    c.setTimeInMillis(newtime.getTime)
    c.add(Calendar.DATE, 1 * n)
    DateFormatUtils.format(c.getTime.getTime, pattern)
  }

  private val case_no = Pattern.compile(".*([（\\(]\\d{4}[）\\)].*?号).*")

  def caseNo(s: String): String = {
    if (StringUtils.isBlank(s)) return ""
    val s1 = s.replaceAll("(\\r\\n|\\r|\\n|\\n\\r|\\s+)", "")
    val matcher = case_no.matcher(s1)
    if (matcher.find) {
      return matcher.replaceAll("$1").replaceAll("\\(", "（").replaceAll("\\)", "）")
    }
    ""
  }

  private val replace_char = "[^\\u4e00-\\u9fa5a-zA-Z\\(\\)（）]+".r

  def replaceChar(s: String) = {
    if (StringUtils.isNotBlank(s)) {
      val arr =
        s.replaceAll("、", ",")
          .replaceAll(";", ",")
          .replaceAll("，", ",")
          .replaceAll(" ", ",")
          .replaceAll("。", ",")
          .replaceAll("；", ",")
          .replaceAll("：", ",")
          .replaceAll("\\s+", ",").split(",")

      val list = arr.toSet.filter(_.length > 1).toList
      if (list.nonEmpty) {
        val sb = new StringBuilder
        for (a <- list) {
          sb.append(a).append(",")
        }
        sb.substring(0, sb.lastIndexOf(",")) toString()
      } else {
        ""
      }
    } else {
      ""
    }
  }

  def BKDRHash(str: String): Long = {
    val seed: Long = 1313131313 // 31 131 1313 13131 131313 etc..
    var hash: Long = 0
    for (i <- 0 to str.length - 1) {
      hash = hash * seed + str.charAt(i)
      hash = hash & 0x7FFFFFFFFFFFFFFFL
    }
    return hash
  }

  def nameJudge(name: String, yg_name: String, bg_name: String): String = {
    if (StringUtils.isNotBlank(name)) {
      if (StringUtils.isNotBlank(yg_name) && yg_name.contains(name)) {
        return "y"
      } else if (StringUtils.isNotBlank(bg_name) && bg_name.contains(name)) {
        return "b"
      }
    }
    ""
  }

  def caseType(caseNO: String): String = {
    if (StringUtils.isNotBlank(caseNO)) {
      if (caseNO.contains("辖")) {
        return "管辖案件"
      } else if (caseNO.contains("民")) {
        return "民事案件"
      } else if (caseNO.contains("执")) {
        return "执行案件"
      } else if (caseNO.contains("行")) {
        return "行政案件"
      } else if (caseNO.contains("刑")) {
        return "刑事案件"
      } else if (caseNO.contains("赔")) {
        return "赔偿案件"
      }
    }
    "其它案件"
  }

  def caseStage(caseNo: String): String = {
    if (StringUtils.isNotBlank(caseNo)) {
      if (StrUtil.containsAny(caseNo, "破申", "商申")) {
        return "其它"
      }
      if (StrUtil.containsAny(caseNo, "监", "抗", "再", "申", "提", "再")) {
        return "再审"
      }
      if (StrUtil.containsAny(caseNo, "初")) {
        return "一审"
      } else if (StrUtil.containsAny(caseNo, "终")) {
        return "二审"
      } else if (StrUtil.containsAny(caseNo, "执")) {
        return "执行"
      }
    }
    "其它"
  }

  def lastStage(s: String): String = {
    if (StringUtils.isNotBlank(s)) {
      val arr = s.split(" ")
      if (StrUtil.containsAny(s, "执")) {
        return arr.filter(StrUtil.containsAny(_, "执")).head
      }
      if (StrUtil.containsAny(s, "再")) {
        return arr.filter(StrUtil.containsAny(_, "再")).head
      }
      if (StrUtil.containsAny(s, "二")) {
        return arr.filter(StrUtil.containsAny(_, "二")).head
      }
      if (StrUtil.containsAny(s, "一")) {
        return arr.filter(StrUtil.containsAny(_, "一")).head
      }
      return arr.head
    }
    "其它"
  }

  def title(ygname: String, bgname: String, reason: String): String = {
    Seq(replaceChar(ygname), replaceChar(bgname), reason).filter(s => StringUtils.isNotBlank(s)).mkString(",")
  }

  def trimBlack(s: String): String = {
    var r = ""
    if (StringUtils.isNotBlank(s)) {
      r = s.split(",").filter(StringUtils.isNotBlank(_)).mkString(",")
    }
    r
  }

  def label(s: String): String = {
    var r = ""
    if (StringUtils.isNotBlank(s)) {
      r = s match {
        case "0" => "裁判文书" //企业
        case "1" => "开庭公告" //企业
        case "2" => "法院公告" //企业
        case "3" => "失信人" //企业
        case "4" => "送达公告" //企业
        case "5" => "限高" //企业
        case "6" => "终本" //企业
        case "7" => "被执行人" //企业
        case "8" => "立案信息" //企业
        case "9" => "失信人" //人
        case "10" => "被执行人" //人
        case "11" => "限高" //人
        case "12" => "终本" //人
        case _ => ""
      }
    }
    r
  }


  /**
   * 全角字符转半角字符
   *
   * @param str
   * @return
   */
  def toDBC(str: String): String = {
    val c = str.toCharArray
    for (i <- 0 until c.length) {
      if (c(i) == 12288) {
        c(i) = 32.toChar
      }
      if (c(i) > 65280 && c(i) < 65375) c(i) = (c(i) - 65248).toChar
    }
    new String(c)
  }

  private val case_pat = ".*([（\\(]\\d{4}[）\\)][^号]*号?).*".r
  private val year_pat = "(\\d{4}?)年".r

  /**
   * 案号格式规整
   *
   * @param str
   * @return
   */
  def case_no_trim(str: String): String = {
    if (StringUtils.isBlank(str)) return null
    var case_no = toDBC(str)
    case_no = case_no.replace(" ", "");
    if (case_no.length < 8) return null
    case_no = year_pat.replaceAllIn(case_no, "\\($1\\)")
    case_no = case_no.replace("（", "(")
      .replace("）", ")")
    if (case_pat matches case_no) {
      case_pat.replaceAllIn(case_no, "$1")
    } else null
  }

  def sortString(s: String): String = {
    var r = ""
    if (StringUtils.isNotBlank(s)) {
      r = s.split("\\001").sorted.mkString("")
    }
    r
  }

  /**
   * 身份证号格式统一
   *
   * @param str
   * @return
   */
  def id_card_trim(str: String): String = {
    if (StringUtils.isNotBlank(str)) {
      return s"${str.substring(0, 10)}****${str.substring(str.length - 4)}".toUpperCase
    }
    null
  }

  private val id_card_pattern = "^[1-9]\\d{5}(19|20)\\d{2}((0[1-9])|(1[0-2])|([0-1]\\*{1,2})|\\*{2})(([0-2][1-9])|10|20|30|31|\\*{2})\\d{3}[0-9Xx]$".r

  private lazy val maxYear = BaseUtil.nowDate(pattern = "yyyy").toInt

  def is_id_card(str: String): Boolean = {
    if (StringUtils.isEmpty(str))
      return false

    if (id_card_pattern matches str) {
      val d = str.substring(6, 10).toInt
      d <= maxYear
    } else
      false
  }

  val pat = "(?<province>[^省]+省|.+自治区)?(?<city>[^自治州]+自治州|[^市]+市|[^盟]+盟|[^地区]+地区|.+区划)?(?<district>[^市]+市|[^县]+县|[^旗]+旗|.+区)?".r

  /**
   * 提取省市区
   *
   * @param addr
   * @return
   */
  def parseAddress(addr: String): (String, String, String) = {
    val matcher = pat.pattern.matcher(addr)
    if (!matcher.find()) {
      return null
    }
    val province = matcher.group("province")
    val city = matcher.group("city")
    val district = matcher.group("district")
    (province, city, district)
  }

  def main(args: Array[String]): Unit = {
//    println(title("xx", null, "reason"))
//    println(parseAddress("大石桥市人民法院"))
//    println(case_no_trim("（2015）怀执字第03601号号"))
//    val seq = Seq("1", "3", "2", "7").mkString("\001")
//    println(sortString(seq))
    println(is_id_card("4111111999****062x"))
  }

}
